@@ -1,21 +1,14 @@
 use strict;
 use warnings;
 package Data::Rx;
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 
 # ABSTRACT: perl implementation of Rx schema system
 
 use Data::Rx::Util;
-use Module::Pluggable::Object;
+use Data::Rx::TypeBundle::Core;
 
 
-sub __built_in_prefixes {
-  return (
-    ''      => 'tag:codesimply.com,2008:rx/core/',
-    '.meta' => 'tag:codesimply.com,2008:rx/meta/',
-  );
-}
-
 sub _expand_uri {
   my ($self, $str) = @_;
   return $str if $str =~ /\A\w+:/;
@@ -39,22 +32,26 @@ sub new {
   $arg ||= {};
   $arg->{prefix} ||= {};
 
+  my @plugins = @{ $arg->{type_plugins} || [] };
+  unshift @plugins, $class->core_bundle unless $arg->{no_core_bundle};
+
   my $self = {
-    prefix  => {
-      $class->__built_in_prefixes,
-      %{ $arg->{prefix} },
-    },
-    handler => {},
+    prefix  => { },
+    handler => { },
   };
 
   bless $self => $class;
 
-  my @plugins = $self->core_type_plugins;
+  PLUGIN: while (my $plugin = shift @plugins) {
+    if ($plugin->isa('Data::Rx::TypeBundle')) {
+      my %pairs = $plugin->prefix_pairs;
+      $self->add_prefix($_ => $pairs{ $_ }) for keys %pairs;
 
-  $self->register_type_plugin($_) for @plugins;
+      unshift @plugins, $plugin->type_plugins;
+      next PLUGIN;
+    }
 
-  if ($arg->{type_plugins}) {
-    $self->register_type_plugin($_) for @{ $arg->{type_plugins} };
+    $self->register_type_plugin($plugin);
   }
 
   return $self;
@@ -84,19 +81,34 @@ sub make_schema {
 sub register_type_plugin {
   my ($self, $plugin) = @_;
 
-  $self->{handler}{ $plugin->type_uri } = $plugin;
+  my $uri = $plugin->type_uri;
+
+  Carp::confess("a type plugin is already registered for $uri")
+    if $self->{handler}{ $uri };
+    
+  $self->{handler}{ $uri } = $plugin;
 }
 
 
+sub add_prefix {
+  my ($self, $name, $base) = @_;
+
+  Carp::confess("the prefix $name is already registered")
+    if $self->{prefix}{ $name };
+
+  $self->{prefix}{ $name } = $base;
+}
+
+sub core_bundle {
+  return 'Data::Rx::TypeBundle::Core';
+}
+
 sub core_type_plugins { 
   my ($self) = @_;
 
-  my $mpo = Module::Pluggable::Object->new(
-    search_path => 'Data::Rx::CoreType',
-    require     => 1,
-  );
+  Carp::cluck("core_type_plugins deprecated; use Data::Rx::TypeBundle::Core");
 
-  my @plugins = $mpo->plugins;
+  Data::Rx::TypeBundle::Core->type_plugins;
 }
 
 1;
@@ -111,7 +123,7 @@ Data::Rx - perl implementation of Rx schema system
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -151,8 +163,9 @@ This returns a new Data::Rx object.
 
 Valid arguments are:
 
-    prefix - optional; a hashref of prefix strings and values for type shorthand
-    type_plugins - optional; an arrayref of type plugins
+    prefix        - optional; a hashref of prefix pairs for type shorthand
+    type_plugins  - optional; an arrayref of type or type bundle plugins
+    no_core_types - optional; if true, core type bundle is not loaded
 
 The prefix hashref should look something like this:
 
@@ -175,9 +188,13 @@ given Rx input.
 Given a type plugin, this registers the plugin with the Data::Rx object.
 Plugins must have a C<type_uri> method and a C<new_checker> method.
 
-=head2 core_type_plugins {
+=head2 add_prefix
 
-This method returns a list of the plugins for the core Rx types.
+    $rx->add_prefix($name => $prefix_string);
+
+For example:
+
+    $rx->add_prefix('.meta' => 'tag:codesimply.com,2008:rx/meta/');
 
 =head1 AUTHOR
 
@@ -185,7 +202,7 @@ This method returns a list of the plugins for the core Rx types.
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2008 by Ricardo SIGNES.
+This software is copyright (c) 2009 by Ricardo SIGNES.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as perl itself.
